#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------------
# Get-PSN-AccountID v2 by Leeful
# 
# A python 3 script to get your PSN Account ID to use with PS4OfflineAccountActivator.exe
# https://github.com/charlyzard/PS4OfflineAccountActivator
# 
# If you need the ps4debug.bin you can download it from here: 
# https://github.com/ChendoChap/Playstation-4-Save-Mounter
# 
# Credit to thestr4ng3r for the chiaki script from which this was based on.
# -----------------------------------------------------------------------------------------

import sys
if sys.version_info < (3, 0, 0):
	print("DO NOT use Python 2.\nEVER.\nhttps://pythonclock.org")
	exit(1)

import platform
oldexit = exit
def exit(code):
	if platform.system() == "Windows":
		import atexit
		input("Press Enter to exit.")
	oldexit(code)

if sys.stdout.encoding.lower() == "ascii":
	import codecs
	sys.stdout = codecs.getwriter('utf-8')(sys.stdout.buffer)
	sys.stderr = codecs.getwriter('utf-8')(sys.stderr.buffer)

try:
	import requests
except ImportError as e:
	print(e)
	if platform.system() == "Windows":
		from distutils.util import strtobool
		a = input("The requests module is not available. Should we try to install it automatically using pip? [y/n] ")
		while True:
			try:
				a = strtobool(a)
				break
			except ValueError:
				a = input("Please answer with y or n: ")
		if a == 1:
			import subprocess
			subprocess.call([sys.executable, "-m", "pip", "install", "requests"])
		else:
			exit(1)
	else:
		print("\"requests\" is not available. Install it with pip or your distribution's package manager.")
		exit(1)

import requests

from urllib.parse import urlparse, parse_qs, quote, urljoin
import pprint
import webbrowser

# Remote Play Windows Client
CLIENT_ID = "ba495a24-818c-472b-b12d-ff231c1b5745"
CLIENT_SECRET = "mvaiZkRsAsI1IBkY"

LOGIN_URL = "https://auth.api.sonyentertainmentnetwork.com/2.0/oauth/authorize?service_entity=urn:service-entity:psn&response_type=code&client_id={}&redirect_uri=https://remoteplay.dl.playstation.net/remoteplay/redirect&scope=psn:clientapp&request_locale=en_US&ui=pr&service_logo=ps&layout_type=popup&smcid=remoteplay&prompt=always&PlatformPrivacyWs1=minimal&".format(CLIENT_ID)
TOKEN_URL = "https://auth.api.sonyentertainmentnetwork.com/2.0/oauth/token"

webbrowser.open_new(LOGIN_URL)

print(" A PSN Sign IN page should now be open in your browser.")
print()
print(" Please sign in and when the page shows \"redirect\", copy the URL from the address bar and paste it here:")
print()
code_url_s = input("> ")
code_url = urlparse(code_url_s)
query = parse_qs(code_url.query)
if "code" not in query or len(query["code"]) == 0 or len(query["code"][0]) == 0:
	print("URL did not contain code parameter")
	exit(1)
code = query["code"][0]
print() 

api_auth = requests.auth.HTTPBasicAuth(CLIENT_ID, CLIENT_SECRET)
body = "grant_type=authorization_code&code={}&redirect_uri=https://remoteplay.dl.playstation.net/remoteplay/redirect&".format(code)

token_request = requests.post(TOKEN_URL,
	auth = api_auth,
	headers = { "Content-Type": "application/x-www-form-urlencoded" },
	data = body.encode("ascii"))

if token_request.status_code != 200:
	print("Request failed with code {}:\n{}".format(token_request.status_code, token_request.text))
	exit(1)

token_json = token_request.json()
if "access_token" not in token_json:
	print("\"access_token\" is missing in response JSON:\n{}".format(token_request.text))
	exit(1)
token = token_json["access_token"]

account_request = requests.get(TOKEN_URL + "/" + quote(token), auth = api_auth)

if account_request.status_code != 200:
	print("Request failed with code {}:\n{}".format(account_request.status_code, account_request.text))
	exit(1)

account_info = account_request.json()

if "user_id" not in account_info:
	print("\"user_id\" is missing in response or not a string")
	exit(1)

user_id = int(account_info["user_id"])
b = user_id
result = '{0:04b}'.format(b)
ACC_ID = "{0:0>16x}".format(int(result, 2))

import os
os.system('cls' if os.name == 'nt' else 'clear')

print("Your PSN Account ID is:")
print(ACC_ID)
print()
print("Enter this Account ID in PS4OfflineAccountActivator.exe")
print()
exit(0)